"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const observable_1 = require("@reactive-js/observable");
const event_1 = require("./event");
const locationEquals = (a, b) => a === b ||
    (a.path === b.path && a.query === b.query && a.fragment === b.fragment);
const chopLeadingChar = (str) => str.length > 0 ? str.substring(1) : "";
const getCurrentLocation = () => {
    const path = window.location.pathname;
    const query = chopLeadingChar(window.location.search);
    const fragment = chopLeadingChar(window.location.hash);
    return { path, query, fragment };
};
const historyPushState = (newLocation) => {
    const currentLocation = getCurrentLocation();
    if (!locationEquals(currentLocation, newLocation)) {
        const { path, query, fragment } = newLocation;
        let uriString = path;
        uriString = query.length > 0 ? `${uriString}?${query}` : uriString;
        uriString = fragment.length > 0 ? `${uriString}#${fragment}` : uriString;
        window.history.pushState(undefined, "", uriString);
    }
};
const getCurrentLocationStateUpdater = (_) => {
    const uri = getCurrentLocation();
    return (_) => uri;
};
const applyUpdaterReducer = (acc, updater) => updater(acc);
class LocationStoreResourceImpl {
    constructor(scheduler) {
        this.subject = observable_1.createSubject();
        const initialLocation = getCurrentLocation();
        const popstateObservable = observable_1.pipe(event_1.fromEvent(window, "popstate", getCurrentLocationStateUpdater), observable_1.onNext(next => this.subject.next(next)), observable_1.ignoreElements());
        const currentLocationObservable = observable_1.pipe(this.subject, observable_1.scan(applyUpdaterReducer, initialLocation), observable_1.onNext(historyPushState), observable_1.startWith(initialLocation));
        this.observable = observable_1.pipe(observable_1.merge(popstateObservable, currentLocationObservable), observable_1.share(scheduler, 1));
        this.subject.add(observable_1.connect(this.observable, scheduler));
    }
    get isDisposed() {
        return this.subject.isDisposed;
    }
    add(disposable, ...disposables) {
        this.subject.add(disposable, ...disposables);
    }
    dispatch(updater) {
        this.subject.next(updater);
    }
    dispose() {
        this.subject.dispose();
    }
    remove(disposable, ...disposables) {
        this.subject.remove(disposable, ...disposables);
    }
    subscribe(subscriber) {
        this.observable.subscribe(subscriber);
    }
}
exports.createLocationStoreResource = (scheduler) => new LocationStoreResourceImpl(scheduler);
//# sourceMappingURL=locationStore.js.map