"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const disposable_1 = require("@reactive-js/disposable");
const schedulers_1 = require("@reactive-js/schedulers");
const performance = window.performance;
const Date = window.Date;
const setTimeout = window.setTimeout;
const clearTimeout = window.clearTimeout;
const yieldInterval = 5;
const maxYieldInterval = 300;
const now = typeof performance === "object" && typeof performance.now === "function"
    ? () => performance.now()
    : () => Date.now();
let startTime = 0;
const shouldYield = navigator !== undefined &&
    navigator.scheduling !== undefined &&
    navigator.scheduling.isInputPending !== undefined
    ? () => {
        const currentTime = now();
        const deadline = startTime + yieldInterval;
        const maxDeadline = startTime + maxYieldInterval;
        const inputPending = navigator.scheduling.isInputPending();
        return ((currentTime >= deadline && inputPending) ||
            currentTime >= maxDeadline);
    }
    : () => now() >= startTime + yieldInterval;
let channel = undefined;
const scheduleImmediate = (continuation, disposable) => {
    channel = channel || new MessageChannel();
    channel.port1.onmessage = () => {
        if (!disposable.isDisposed) {
            continuation();
        }
    };
    channel.port2.postMessage(null);
};
const schedule = (continuation, delay = 0) => {
    const scheduledContinuation = () => {
        startTime = now();
        let result = continuation;
        while (result !== undefined) {
            result = result();
        }
    };
    const disposable = disposable_1.createDisposable();
    if (delay >= 4) {
        const timeout = setTimeout(scheduledContinuation, delay);
        disposable.add(() => clearTimeout(timeout));
    }
    else {
        scheduleImmediate(scheduledContinuation, disposable);
    }
    return disposable;
};
const schedulerHost = {
    get now() {
        return now();
    },
    get shouldYield() {
        return shouldYield();
    },
    schedule,
};
let priorityScheduler = undefined;
exports.createSchedulerWithPriority = (priority) => {
    priorityScheduler =
        priorityScheduler || schedulers_1.createPrioritySchedulerResource(schedulerHost);
    return schedulers_1.createSchedulerWithPriority(priorityScheduler, priority);
};
//# sourceMappingURL=scheduler.js.map