import { pipe, createSubject, connect, ignoreElements, share, onNext, merge, scan, startWith, } from "@reactive-js/observable";
import { fromEvent } from "./event";
const locationEquals = (a, b) => a === b ||
    (a.path === b.path && a.query === b.query && a.fragment === b.fragment);
const chopLeadingChar = (str) => str.length > 0 ? str.substring(1) : "";
const getCurrentLocation = () => {
    const path = window.location.pathname;
    const query = chopLeadingChar(window.location.search);
    const fragment = chopLeadingChar(window.location.hash);
    return { path, query, fragment };
};
const historyPushState = (newLocation) => {
    const currentLocation = getCurrentLocation();
    if (!locationEquals(currentLocation, newLocation)) {
        const { path, query, fragment } = newLocation;
        let uriString = path;
        uriString = query.length > 0 ? `${uriString}?${query}` : uriString;
        uriString = fragment.length > 0 ? `${uriString}#${fragment}` : uriString;
        window.history.pushState(undefined, "", uriString);
    }
};
const getCurrentLocationStateUpdater = (_) => {
    const uri = getCurrentLocation();
    return (_) => uri;
};
const applyUpdaterReducer = (acc, updater) => updater(acc);
class LocationStoreResourceImpl {
    constructor(scheduler) {
        this.subject = createSubject();
        const initialLocation = getCurrentLocation();
        const popstateObservable = pipe(fromEvent(window, "popstate", getCurrentLocationStateUpdater), onNext(next => this.subject.next(next)), ignoreElements());
        const currentLocationObservable = pipe(this.subject, scan(applyUpdaterReducer, initialLocation), onNext(historyPushState), startWith(initialLocation));
        this.observable = pipe(merge(popstateObservable, currentLocationObservable), share(scheduler, 1));
        this.subject.add(connect(this.observable, scheduler));
    }
    get isDisposed() {
        return this.subject.isDisposed;
    }
    add(disposable, ...disposables) {
        this.subject.add(disposable, ...disposables);
    }
    dispatch(updater) {
        this.subject.next(updater);
    }
    dispose() {
        this.subject.dispose();
    }
    remove(disposable, ...disposables) {
        this.subject.remove(disposable, ...disposables);
    }
    subscribe(subscriber) {
        this.observable.subscribe(subscriber);
    }
}
export const createLocationStoreResource = (scheduler) => new LocationStoreResourceImpl(scheduler);
//# sourceMappingURL=locationStore.js.map